import matplotlib.pyplot as plt
from openpyxl import load_workbook
from openpyxl.worksheet import Worksheet

from performance import SHEET_NAME


def plt_export_chart(excel_file, output):
    excel = load_workbook(excel_file)
    table = excel[SHEET_NAME]
    assert isinstance(table, Worksheet)
    rows = table.max_row

    datax = []
    datay = []
    i = 1
    for row in range(2, rows + 1):
        datax.append(table.cell(row, 1).value)
        datay.append(table.cell(row, 2).value)
        i += 1

    # 设置输出的图片大小
    figsize = 14, 9
    figure, ax = plt.subplots(figsize=figsize)

    plt.title("Total Memory Summary")
    plt.ylabel("Memory(KB)")
    plt.xlabel("Time")
    plt.plot(datax, datay, color='Red')
    plt.xticks(datax, __get_ticks(datax), rotation='vertical')
    plt.tick_params(labelsize=14)
    labels = ax.get_xticklabels() + ax.get_yticklabels()
    [label.set_fontname('Times New Roman') for label in labels]
    plt.savefig(output)
    plt.close('all')


def plt_export_history(pkg: str, histories: list, output: str):
    data_avg = []
    data_max = []
    datax = []

    for history in histories:
        if history.pkg != pkg:
            continue
        if not history.success:
            continue
        data_avg.append(history.avg)
        data_max.append(history.max)
        datax.append(history.date)

    # 设置输出的图片大小
    figsize = 14, 9
    figure, ax = plt.subplots(figsize=figsize)

    plt.title("History Memory Summary")
    plt.ylabel("Memory(KB)")
    plt.xlabel("Date")
    l1, = plt.plot(datax, data_max, color='Red')
    l2, = plt.plot(datax, data_avg, color='Blue')
    plt.xticks(datax, datax, rotation='vertical')
    plt.legend(handles=[l1, l2], labels=['max', 'avg'], loc='upper center', frameon=False)
    plt.tick_params(labelsize=12)
    labels = ax.get_xticklabels() + ax.get_yticklabels()
    [label.set_fontname('Times New Roman') for label in labels]
    plt.savefig(output)
    plt.close('all')


def __get_ticks(xs: list):
    show_len = 30
    count = len(xs)
    # 30取1
    if count < show_len:
        return xs
    length = int(count / show_len + 1)
    data = [xs[0]]
    for i in range(1, len(xs) - 1):
        if i % length == 0:
            data.append(xs[i])
        else:
            data.append('')
    data.append(xs[-1])
    return data




import os
import sys
import threading
from getopt import getopt

import performance

from performance.android import AndroidApp

from performance import VERSION, android, properties, config
from performance import android_device
from performance.android_device import Device
from performance.config import WORK_DIR
from performance.log import tip


def usage():
    print("Performance Test Kit")
    print("usage: ptk [-d device] [-o output] [-t duration] [-p property_file] [-s] [-v] [-h] <app_package_name>")
    print()
    print("-h 打印此帮助")
    print("-d 设备号，可以通过adb devices命令拿到")
    print("-v 打印ptk版本号")
    print("-o 数据，日志的输出目录")
    print("-s 停止, 后面需要跟app包名")
    print("-t 时间(h), 可以是整数或者小数作为参数, 比如0.1,0.5,1,2,5,8..., 默认值是10")
    print("-p 属性文件,默认取 ~/ptk/ptk.properties")


def version():
    return '.'.join(str(v) for v in VERSION)


def main():
    #
    devices = None
    output = WORK_DIR
    series = None
    stop = False

    hours = 10.0
    try:
        opts, args = getopt(sys.argv[1:], "d:o:t:p:shv")
        for op, value in opts:
            if op == '-o':
                output = value
            elif op == '-d':
                series = value
            elif op == '-h':
                usage()
                exit(0)
            elif op == '-v':
                print("pkt: " + version())
                exit(0)
            elif op == '-t':
                hours = float(value)
            elif op == '-s':
                stop = True
            elif op == '-p':
                if not os.path.exists(value):
                    tip("配置文件不存在: %s " % value)
                    exit(1)
                config.ptk_properties = properties.parse(value)
        if len(args) < 1:
            usage()
            exit(1)
        pkg = args[0]
        if pkg is None or pkg == '':
            usage()
            exit(0)

        devices = android_device.get_devices()

        if not len(devices):
            tip("error: no devices connected !!")
            return
        for device in devices:
            if output is not None:
                device.output = output
            if series is not None and device.series == series:
                devices = [device]
                break

        tip("检测应用: " + pkg)
        tip("检测时长: %s(s)" % str(int(hours * 60 * 60)))
        tip("输出目录: " + output)
        tip("检测设备: " + str(devices))
        tip("属性配置: " + str(os.path.abspath(config.ptk_properties.file_name)))
        tip("")

        if stop:
            for device in devices:
                device.pkg = pkg
                device.app = AndroidApp(pkg, series=device.series)
                device.stop()
                return

        device_threads = []
        for device in devices:
            assert isinstance(device, Device)
            device.pkg = pkg
            device.duration = hours
            device_thread = DeviceThread(device)
            device_thread.start()
            device_threads.append(device_thread)

        for thread in device_threads:
            time_out = hours * 60 * 60
            thread.join(timeout=time_out)

    except Exception as e:
        if devices is None:
            return
        for device in devices:
            device.app.success = False
        raise e
    finally:
        if devices is None:
            return
        for device in devices:
            device.app.run = False


class DeviceThread(threading.Thread):

    def __init__(self, device):
        super().__init__()
        assert isinstance(device, Device)
        self.device = device
        self.setDaemon(False)

    def run(self):
        try:
            self.device.start()
        except Exception as e:
            raise e
        finally:
            android.stop(self.device.app)



from sqlite3 import Cursor

type_dict = {'str': 'text', 'bool': 'int', 'float': 'real', 'int': 'int'}


def create_table_if_need(obj: object, conn):
    """
        根据任意对象obj成员对象来创建表
        obj的成员对象必须赋初始值
    """
    table_name = obj.__class__.__name__
    cursor = conn.cursor()

    sql = 'create table if not exists %s ( id  integer primary key autoincrement, ' % table_name
    for attr in dir(obj):
        if attr.startswith('__') and attr.endswith('__'):
            continue
        if attr == 'id':
            continue
        cls_type = obj.__getattribute__(attr).__class__.__name__
        sql_type = type_dict.get(cls_type)
        sql += attr + " " + sql_type + ","

    sql = sql.strip().rstrip(',') + ");"
    cursor.execute(sql)
    conn.commit()
    cursor.close()


def insert(obj: object, conn):
    table_name = obj.__class__.__name__

    cursor = conn.cursor()
    attrs = [attr for attr in dir(obj) if not attr.startswith('__') and not attr.endswith('__')]
    attrs_name = ','.join(attrs)

    values = ''
    for attr in attrs:
        attr_value = obj.__getattribute__(attr)
        if attr_value.__class__ == str:
            values += "'%s'," % attr_value
        elif attr_value.__class__ == bool:
            if attr_value:
                values += str(1)
            else:
                values += str(0)
            values += ","
        else:
            values += str(attr_value) + ","
    attrs_value = values[0: len(values) - 1]
    sql = 'insert into %s (%s) values (%s)' % (table_name, attrs_name, attrs_value)

    try:
        cursor.execute(sql)
    except Exception as e:
        raise e

    conn.commit()
    cursor.close()
    return True


def query(obj: object, conn):
    """
    搜索
    :param conn:
    :param obj: 搜索对象
    :return: 搜索结果
    """
    table_name = obj.__class__.__name__

    cursor = conn.cursor()

    cursor.execute("select * from %s where id = %d" % (table_name, obj.id))

    result = cursor.fetchone()
    cursor.close()
    conn.commit()
    return result


def query_all(obj: object, conn):
    """
        搜索
        :param conn:
        :param obj: 搜索对象
        :return: 搜索结果
        """
    table_name = obj.__class__.__name__

    cursor = conn.cursor()

    cursor.execute("select * from %s" % table_name)
    assert isinstance(cursor, Cursor)
    data = cursor.fetchall()
    result = []
    for item in data:
        new_obj = obj.__class__()
        attrs = [attr for attr in dir(obj) if not attr.startswith('__') and not attr.endswith('__')]
        for i in range(len(attrs)):
            attr = attrs[i]
            new_obj.__setattr__(attr, item[i + 1])
        result.append(new_obj)
    cursor.close()
    conn.commit()
    return result
